<?php

include "BinaryStream.php";

class B4XSerializator{
	const T_NULL = 0;
	const T_STRING = 1;
	const T_SHORT = 2;
	const T_INT = 3;
	const T_LONG = 4;
	const T_FLOAT = 5;
	const T_DOUBLE = 6;
	const T_BOOLEAN = 7;
	const T_BYTE = 10;
	const T_CHAR = 14;
	const T_MAP = 20;
	const T_LIST = 21;
	const T_NSARRAY = 22;
	const T_NSDATA = 23;
	const T_TYPE = 24;
	
	protected $memorystream;
	protected $bs;
	
	public function __construct(){
		
	}
	
	public function ConvertBytesToObject($bytes) {
		$this->memorystream = fopen('php://memory', 'w+');
		$this->bs = new BinaryStream($this->memorystream, BinaryStream::CREATE);
		$this->bs->setEndian(BinaryStream::LITTLE);
		fwrite($this->memorystream, gzdecode($bytes));
		rewind($this->memorystream);
		return $this->readObject();
	}
	
	public function ConvertObjectToBytes($object) {
		$this->memorystream = fopen('php://memory', 'w+');
		$this->bs = new BinaryStream($this->memorystream, BinaryStream::CREATE);
		$this->bs->setEndian(BinaryStream::LITTLE);
		
		$this->writeObject($object);
		rewind($this->memorystream);
		return gzencode(stream_get_contents($this->memorystream), -1, FORCE_DEFLATE);
	}
	
	private function writeObject($o) {
		if (is_null($o))
			$this->writeByte(self::T_NULL);
		elseif (is_integer($o)) {
			$this->writeByte(self::T_INT);
			$this->writeInt($o);
		} elseif (is_float($o)) {
			$this->writeByte(self::T_FLOAT);
			$this->writeFloat($o);
		} elseif (is_double($o)) {
			$this->writeByte(self::T_DOUBLE);
			$this->writeDouble($o);
		} elseif (is_bool($o)) {
			$this->writeByte(self::T_BOOLEAN);
			$this->writeByte($o?1:0);
		} elseif (is_string($o)) {
			$this->writeByte(self::T_STRING);
			$this->writeInt(strlen($o));
			$this->writeStr($o);
		}
	}
	
	private function writeByte($b) {
		$this->bs->writeInteger($b, 8);
	}
	
	private function writeInt($i) {
		$this->bs->writeInteger($i, 32);
	}
	
	private function writeStr($str) {
		$this->bs->writeString($str);
	}
	
	private function writeFloat($n) {
		$this->bs->writeFloat($n, 32);
	}
	
	private function writeDouble($n) {
		$this->bs->writeDouble($n, 64);
	}
	
	private function readObject() {
		$t = $this->readByte();
		switch ($t) {
			case self::T_NULL:
				return null;
			case self::T_INT:
				return $this->readInt();
			case self::T_SHORT:
				return $this->readShort();
			case self::T_FLOAT:
				return $this->readFloat();
			case self::T_DOUBLE:
				return $this->readDouble();
			case self::T_BOOLEAN:
				return $this->readByte()==1?true:false;
			case self::T_STRING:
				$len = $this->readInt();
				return $this->readStr($len);
		}
	}
	
	private function readByte() {
		return $this->bs->readInteger(8);
	}
	
	private function readShort(){
		return $this->bs->readInteger(16);
	}
	
	private function readInt(){
		return $this->bs->readInteger(32);
	}
	
	private function readStr($len){
		return $this->bs->readString($len);
	}
	
	private function readFloat(){
		return $this->bs->readFloat();
	}
	
	private function readDouble(){
		return $this->bs->readDouble();
	}
}


